use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::CheckBreaks 0.012

use Test::More 0.88;

SKIP: {
    skip 'no conflicts module found to check against', 1;
}

pass 'no x_breaks data to check';

done_testing;
